@@ -5,16 +5,21 @@ import java.util.HashMap;
 import java.util.List;
 
 import org.gcube.portlets.user.gisviewer.client.Constants;
-import org.gcube.portlets.user.gisviewer.client.GisViewerPanel;
+import org.gcube.portlets.user.gisviewer.client.OpenLayersMapParameters;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.GisViewerBaseLayerInterface;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.LayerItem;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.MapViewInfo;
 import org.gcube.portlets.user.gisviewer.client.commons.beans.TransectParameters;
+import org.gcube.portlets.user.gisviewer.client.resources.Images;
 import org.gwtopenmaps.openlayers.client.Bounds;
+import org.gwtopenmaps.openlayers.client.Icon;
 import org.gwtopenmaps.openlayers.client.LonLat;
 import org.gwtopenmaps.openlayers.client.Map;
 import org.gwtopenmaps.openlayers.client.MapOptions;
 import org.gwtopenmaps.openlayers.client.MapWidget;
+import org.gwtopenmaps.openlayers.client.Marker;
+import org.gwtopenmaps.openlayers.client.Pixel;
+import org.gwtopenmaps.openlayers.client.Size;
 import org.gwtopenmaps.openlayers.client.ZIndexBase;
 import org.gwtopenmaps.openlayers.client.control.Control;
 import org.gwtopenmaps.openlayers.client.control.DrawFeature;
@@ -35,24 +40,40 @@ import org.gwtopenmaps.openlayers.client.geometry.Point;
 import org.gwtopenmaps.openlayers.client.handler.RegularPolygonHandler;
 import org.gwtopenmaps.openlayers.client.handler.RegularPolygonHandlerOptions;
 import org.gwtopenmaps.openlayers.client.layer.Layer;
+import org.gwtopenmaps.openlayers.client.layer.Markers;
 import org.gwtopenmaps.openlayers.client.layer.Vector;
 import org.gwtopenmaps.openlayers.client.layer.VectorOptions;
 import org.gwtopenmaps.openlayers.client.layer.WMS;
 import org.gwtopenmaps.openlayers.client.layer.WMSOptions;
 import org.gwtopenmaps.openlayers.client.layer.WMSParams;
+import org.gwtopenmaps.openlayers.client.tile.TileOptions;
 import org.gwtopenmaps.openlayers.client.util.JObjectArray;
 import org.gwtopenmaps.openlayers.client.util.JSObject;
 
 import com.extjs.gxt.ui.client.widget.ContentPanel;
 import com.extjs.gxt.ui.client.widget.layout.FitLayout;
+import com.google.gwt.core.client.Scheduler;
+import com.google.gwt.core.client.Scheduler.ScheduledCommand;
 import com.google.gwt.core.shared.GWT;
 import com.google.gwt.user.client.Element;
 import com.google.gwt.user.client.Window;
 import com.google.gwt.user.client.ui.DockPanel;
 import com.google.gwt.user.client.ui.Widget;
 
+// TODO: Auto-generated Javadoc
+/**
+ * The Class OpenLayersMap.
+ *
+ * @author Francesco Mangiacrapa francesco.mangiacrapa@isti.cnr.it
+ * Feb 10, 2016
+ */
 public class OpenLayersMap {
 
+	/**
+	 *
+	 */
+	public static final String Z_AXIS_NCWMS_KEY_PARAMETER = "ELEVATION";
+	public static final String Z_AXIS_WMS_KEY_PARAMETER = "ELEVATION";
 	protected static final int MAX_ZINDEX = 280;
 	private List<LayerItem> layerItems = new ArrayList<LayerItem>();
 	private Map map;
@@ -63,11 +84,7 @@ public class OpenLayersMap {
 	private HashMap<Layer, LayerItem> mappingLayersToLayerItems = new HashMap<Layer, LayerItem>();
 	private NavToolbar mouseToolBar;
 	private MouseDefaults mouseDefaults;
-	//private OverviewMap overViewMap;
-	//private MouseClickOnMap mouseClickOnMap;
-//	private ToolBarPanelOld toolBarPanel;
 	private int numZoomLevels = 0;
-	//private String projection = "";
 	private Vector vectorLayer;
 	private ZoomBox zoomBox;
 	private boolean clickData = false;
@@ -84,16 +101,30 @@ public class OpenLayersMap {
 	private ToolBarPanel toolBarPanel;
 	private List<GisViewerBaseLayerInterface> baseLayersIntefaces;
 	private TransectParameters transect;
-	
+
 	//ADDED BY FRANCESCO M. TO FIX # support 832
+	/**
+	 * The Class ZIndexBaseGisViewer.
+	 *
+	 * @author Francesco Mangiacrapa francesco.mangiacrapa@isti.cnr.it
+	 * Feb 10, 2016
+	 */
 	public class ZIndexBaseGisViewer extends ZIndexBase{
-		
+
+		/**
+		 * Instantiates a new z index base gis viewer.
+		 *
+		 * @param jsObject the js object
+		 */
 		protected ZIndexBaseGisViewer(JSObject jsObject) {
 			super(jsObject);
 		}
 
 		//Ideally this is initialized from OpenLayers, but it seems that
 		// would require instantiating OpenLayers.Map to copy default Z_INDEX_BASE
+		/**
+		 * Instantiates a new z index base gis viewer.
+		 */
 		public ZIndexBaseGisViewer(){
 			this(JSObject.createJSObject());
 			setForBaseLayers(20);
@@ -103,6 +134,15 @@ public class OpenLayersMap {
 			setForControls(240);
 		}
 
+		/**
+		 * Instantiates a new z index base gis viewer.
+		 *
+		 * @param baseLayerZIndexBase the base layer z index base
+		 * @param overlayZIndexBase the overlay z index base
+		 * @param featureZIndexBase the feature z index base
+		 * @param popupZIndexBase the popup z index base
+		 * @param controlZIndexBase the control z index base
+		 */
 		public ZIndexBaseGisViewer(int baseLayerZIndexBase,
 				int overlayZIndexBase,
 				int featureZIndexBase,
@@ -117,30 +157,88 @@ public class OpenLayersMap {
 		}
 	}
 	// constructor
-	public OpenLayersMap(String w, String h, int numZoomLevels, // projection, transectUrl removed
-			String title, final OpenLayersHandler handler) {
+
+	/**
+	 * Instantiates a new open layers map.
+	 *
+	 * @param w the w
+	 * @param h the h
+	 * @param numZoomLevels the num zoom levels
+	 * @param title the title
+	 * @param baseLayers the base layers
+	 * @param centerMapTo the center map to ex. "To center on Italy" you may pass LonLat(12.45, 42.98);
+	 * @param initZoomTo the init zoom to ex. 6 to set initial level zoom to 6
+	 * @param handler the handler
+	 */
+	public OpenLayersMap(String w, String h, 
+			int numZoomLevels,
+			String title, 
+			List<? extends GisViewerBaseLayerInterface> baseLayers, 
+			final OpenLayersMapParameters openLayersMapParameters, 
+			final OpenLayersHandler handler) {
 
 		this.handler = handler;
 		this.numZoomLevels = numZoomLevels;
 		//this.projection = defaultBounds.;
-
 		// set map options
+		
+
+		//Managing maxzoomlevel parameter TO OL MAP
+		String maxzoomlevel = openLayersMapParameters.getPameter(OpenLayersMapParameters.OL_MAP_PARAM.maxzoomlevel.name());
+		try {
+			if(maxzoomlevel!=null) {
+				int maxZoom = Integer.parseInt(maxzoomlevel);
+				this.numZoomLevels = maxZoom<=Constants.numMAXZoomLevels?maxZoom:Constants.numZoomLevels;
+			}
+		}catch (Exception e) {
+			GWT.log("BAD value: "+maxzoomlevel+" for parameter: "+OpenLayersMapParameters.OL_MAP_PARAM.maxzoomlevel);
+		}
+		
+		//Managing zoom parameter TO OL MAP
+		String paramZoom = openLayersMapParameters.getPameter(OpenLayersMapParameters.OL_MAP_PARAM.zoom.name());
+		try {
+			if(paramZoom!=null) {
+				int zoomTo = Integer.parseInt(paramZoom);
+				setZoomLevel(zoomTo);
+			}
+		}catch (Exception e) {
+			GWT.log("BAD value: "+paramZoom+" for parameter: "+OpenLayersMapParameters.OL_MAP_PARAM.zoom);
+		}
+		
+		//Managing centermap parameter TO OL MAP
+		String centerMapTo = openLayersMapParameters.getPameter(OpenLayersMapParameters.OL_MAP_PARAM.centermap.name());
+		if(centerMapTo!=null) {
+		
+			LonLat lonlat = null;
+			try {
+				String[] theLongLat = centerMapTo.split(",");
+				lonlat = new LonLat(Double.parseDouble(theLongLat[0]), Double.parseDouble(theLongLat[1]));
+				//gisViewerPanel.getOpenLayersMap().setCenterAt(lonlat);
+				
+			}catch (Exception e1) {
+				GWT.log("BAD value: "+centerMapTo+" for parameter: "+OpenLayersMapParameters.OL_MAP_PARAM.centermap);
+			}
+			
+			if(lonlat!=null) {
+				final LonLat theLonlat = lonlat;
+				setCenterAt(theLonlat);
+			}
+		}
+		
+		
 		MapOptions mapOptions = new MapOptions();
 		mapOptions.setControls(new JObjectArray(new JSObject[] {}));
-		mapOptions.setNumZoomLevels(numZoomLevels);
+		mapOptions.setNumZoomLevels(this.numZoomLevels);
 		mapOptions.setProjection(Constants.defaultProjection);
-		
 		mapOptions.setZIndexBase(new ZIndexBaseGisViewer());
-		
 
 		// let's create map widget and map objects
 		mapWidget = new MapWidget(w, h, mapOptions); // map widget
-		map = mapWidget.getMap(); // map js-mapped object		
+		map = mapWidget.getMap(); // map js-mapped object
 		mapWidget.setTitle(title); // set title
 		map.setMaxExtent(defaultBounds);
-
+		//Bounds boundOnItaly = new Bounds(22.5,56.25,28.125,61.875);
 		//map.getJSObject().setProperty("allOverlays", true);
-
 		// set click listener on the map
 		map.addMapClickListener(new MapClickListener() {
 			public void onClick(MapClickEvent mapClickEvent) {
@@ -157,17 +255,17 @@ public class OpenLayersMap {
 					int w = (int) map.getSize().getWidth();
 					int h = (int) map.getSize().getHeight();
 //					handler.clickOnMap(x, y, w, h);
-					
+
 					// ratio -  mapPixelWeight : bboxWeight = 10px : geoRectangleWidth
 					// where 10px is the pixel diameter dimension of the clicked point
-					
+
 					double bboxWidth = Math.abs(getExtent().getLowerLeftX() - getExtent().getUpperRightX());
 					double geoWidth = bboxWidth / w * 10 / 2;
 					double x1 = Math.min(lon+geoWidth, lon-geoWidth);
 					double x2 = Math.max(lon+geoWidth, lon-geoWidth);
 					double y1 = Math.min(lat+geoWidth, lat-geoWidth);
 					double y2 = Math.max(lat+geoWidth, lat-geoWidth);
-					Constants.log("("+(x1)+","+(y1)+")("+(x2)+","+(y2)+")");
+					Constants.log("("+x1+","+y1+")("+x2+","+y2+")");
 					handler.selectBox(x1, y1, x2, y2);
 				}
 			}
@@ -178,12 +276,10 @@ public class OpenLayersMap {
 		mouseToolBar = new NavToolbar();
 		PanZoomBar pamZoomBar = new PanZoomBar();
 		zoomBox = new ZoomBox();
-
 		Element e = mouseDefaults.getJSObject().getPropertyAsDomElement("position");
 		e.setPropertyDouble("x", -300);
 		mouseDefaults.getJSObject().setProperty("position", e);
 
-	
 		VectorOptions vectoroptions = new VectorOptions();
 		vectoroptions.setDisplayInLayerSwitcher(false);
 		vectoroptions.setDisplayOutsideMaxExtent(true);
@@ -191,53 +287,30 @@ public class OpenLayersMap {
 		vectorLayer = new Vector("transet", vectoroptions);
 		map.addLayer(vectorLayer);
 		vectorLayer.setZIndex(MAX_ZINDEX);
-		
-//		addBaseLayers();
 
-		String wmsURI = GisViewerPanel.resources.baseLayer().getText();
+		//adding base layers
+		for (GisViewerBaseLayerInterface baseLayer : baseLayers) {
+			String baseLayerTitle = baseLayer.getTitle();
+			if(baseLayer.isDisplay())
+				addBaseLayerToOpenLayerMap(baseLayerTitle, baseLayer.getName(), baseLayer.getWmsServiceBaseURL());
+		}
 
-		String[] params = wmsURI.split("\\?");
-		
-		if(params.length<2){
-			GWT.log("ERROR: No base layer found!");
-		}
-		
-		String baseLayerURI = params[0];
-		String[] parameter = params[1].split("&");
-		
-		String baseLayerName = "";
-		
-		for (String string : parameter) {
-			if(string.startsWith("layers=")){
-				baseLayerName = string.substring(7);
-				break;
-			}
-		}
-		
-		if(baseLayerURI==null || baseLayerURI.isEmpty() || baseLayerName.isEmpty()){
-			GWT.log("ERROR: No base layer found!");
-		}
-			
-		
-		addBaseLayerToOpenLayerMap("True Marble", baseLayerName, baseLayerURI);
-		
 		addControl(mouseDefaults);
 		addControl(pamZoomBar);
 		addControl(getMousePosition());
 		addControl(zoomBox);
 		addControl(new LayerSwitcher());
-
 		addTransectControl();
 		addBoxControl();
 	}
 
 	/**
-	 * 
+	 * Adds the transect control.
 	 */
 	private void addTransectControl() {
 		FeatureAddedListener listener = new FeatureAddedListener() {
 			public void onFeatureAdded(VectorFeature vf) {
-				
+
 				GWT.log("Transect "+transect);
 				if (transect.getUrl()==null)
 					return;
@@ -272,29 +345,29 @@ public class OpenLayersMap {
 			}
 
 		};
-		
+
 		DrawFeatureOptions drawoptions = new DrawFeatureOptions();
 		drawoptions.onFeatureAdded(listener);
-
 		LineHandler handler = new LineHandler();
-
 		drawLineControl = new DrawFeature(vectorLayer, handler, drawoptions);
 		addControl(drawLineControl);
 	}
 
 
 	/**
-	 * 
+	 * Adds the box control.
 	 */
 	private void addBoxControl() {
 		FeatureAddedListener listener = new FeatureAddedListener() {
 			@Override
 			public void onFeatureAdded(VectorFeature vf) {
+				
 				if (prevVf!=null)
 					vectorLayer.removeFeature(prevVf);
+				
 				prevVf = vf;
 				vectorLayer.setZIndex(MAX_ZINDEX);
-				
+
 				Geometry geo = vf.getGeometry();
                 Bounds bounds = geo.getBounds();
                 double x1 = bounds.getLowerLeftX();
@@ -304,7 +377,7 @@ public class OpenLayersMap {
 				handler.selectBox(x1, y1, x2, y2);
 			}
 		};
-		
+
 		/*
 		DrawFeatureOptions options = new DrawFeatureOptions();
 		options.onFeatureAdded(listener);
@@ -315,7 +388,7 @@ public class OpenLayersMap {
 		drawBoxControl = new DrawFeature(vectorLayer, new RegularPolygonHandler(), options);
 		this.map.addControl(drawBoxControl);
 		*/
-		
+
 		//CHANGED BY FRANCESCO
 		DrawFeatureOptions options = new DrawFeatureOptions();
 		options.onFeatureAdded(listener);
@@ -329,7 +402,9 @@ public class OpenLayersMap {
 	}
 
 	/**
-	 * @return
+	 * Gets the mouse position.
+	 *
+	 * @return the mouse position
 	 */
 	private Control getMousePosition() {
 		// set text mouse position (latitude and longitude)
@@ -355,16 +430,26 @@ public class OpenLayersMap {
 			}
 
 		};
+		
 		MousePositionOptions mpOptions = new MousePositionOptions();
 		mpOptions.setFormatOutput(mpOut);
 		MousePosition mousePosition = new MousePosition(mpOptions);
 		return mousePosition;
 	}
 
+	/**
+	 * Activate zoom out.
+	 */
 	public void activateZoomOut() {
 		mouseToolBar.getJSObject().setProperty("mode", "zoomout");
 	}
 
+	/**
+	 * Adds the layer items.
+	 *
+	 * @param layerItems the layer items
+	 * @param toTop the to top
+	 */
 	public void addLayerItems(List<LayerItem> layerItems, boolean toTop) {
 		if (toTop)
 			for (LayerItem layerItem : layerItems)
@@ -381,13 +466,18 @@ public class OpenLayersMap {
 		this.addLayers(lsArray);
 	}
 
+	/**
+	 * Adds the layers.
+	 *
+	 * @param layers the layers
+	 */
 	private void addLayers(Layer[] layers) {
 
 		for (Layer l : layers) {
 			l.setIsBaseLayer(false);
 			l.getJSObject().setProperty("animationEnabled", true);
 			l.getJSObject().setProperty("displayInLayerSwitcher", false);
-			
+
 			if (l.getName().contentEquals("Google Satellite"))
 				mapping.put("Google Satellite", l.getId());
 			else {
@@ -398,38 +488,40 @@ public class OpenLayersMap {
 
 		map.addLayers(layers);
 		//vectorLayer.setZIndex(MAX_ZINDEX);
-		
-		// OverviewMapOptions overOptions = new OverviewMapOptions();
-		// Layer[] ls = new Layer[1];
-		// ls[0] = layers[0];
-		//
-		// overOptions.setLayers(ls);
-		//
-		// if (Constants.isOverViewMapVisible) {
-		// overViewMap = new OverviewMap();
-		// addControl(overViewMap);
-		// }
 	}
 
-//	public void addToolBarOld(ToolBarPanelOld toolBarPanel) {
-//		this.toolBarPanel = toolBarPanel;
-//	}
-//
 	/**
-	 * @param toolBarPanel2
+	 * Adds the tool bar.
+	 *
+	 * @param toolBarPanel the tool bar panel
 	 */
 	public void addToolBar(ToolBarPanel toolBarPanel) {
 		this.toolBarPanel = toolBarPanel;
 	}
 
+	/**
+	 * Adds the control.
+	 *
+	 * @param control the control
+	 */
 	private void addControl(Control control) {
 		map.addControl(control);
 	}
 
+	/**
+	 * Gets the layer items.
+	 *
+	 * @return the layer items
+	 */
 	public List<LayerItem> getLayerItems() {
 		return layerItems;
 	}
 
+	/**
+	 * Gets the dock panel.
+	 *
+	 * @return the dock panel
+	 */
 	public Widget getDockPanel() {
 		// int zoom=map.getZoomForExtent(map.getMaxExtent(), true);
 		// let's center the map to somewhere and set zoom level to 5
@@ -442,35 +534,78 @@ public class OpenLayersMap {
 
 		if (toolBarPanel != null)
 			dockPanel.add(toolBarPanel, DockPanel.NORTH);
-		
+
 		dockPanel.add(mapWidget, DockPanel.CENTER);
 
 		return dockPanel;
 
 	}
 
+	/**
+	 * Gets the content panel.
+	 *
+	 * @return the content panel
+	 */
 	public ContentPanel getContentPanel() {
 		ContentPanel panel = new ContentPanel();
 		panel.setHeaderVisible(false);
 		panel.setLayout(new FitLayout());
 		panel.add(mapWidget);
 		panel.setTopComponent(toolBarPanel);
-		
+
 		// int zoom=map.getZoomForExtent(map.getMaxExtent(), true);
 		// let's center the map to somewhere and set zoom level to 5
 		LonLat center = new LonLat(0, 0);
 		map.setCenter(center, Constants.openLayersMapDefaultZoom);
 		// map.setCenter(center, zoom);
-
+		
 		return panel;
 	}
+	
+	public void addMarker(LonLat lonLat) {
+		Markers markers = new Markers("Markers");
+	    map.addLayer(markers);
+	   
+	    Size size = new Size(21,25);
+	    Pixel offset = new Pixel(-(size.getWidth()/2), -size.getHeight());
+	    Icon icon = new Icon(Images.mapMarked().createImage().getUrl(), size, offset);
+	    markers.addMarker(new Marker(lonLat,icon));
+	    //markers.addMarker(new Marker(lonLat,icon.clone()));
+	}
 
+	/**
+	 * Sets the opacity.
+	 *
+	 * @param layerItem the layer item
+	 * @param opacity the opacity
+	 */
 	public void setOpacity(LayerItem layerItem, double opacity) {
 		Layer layer = mappingLayerItemsToLayers.get(layerItem);
 		if (layer != null)
 			layer.setOpacity(new Float(opacity).floatValue());
 	}
 
+	/**
+	 * Sets the z axis value.
+	 *
+	 * @param layerItem the layer item
+	 * @param intValue the int value
+	 */
+	public void setZAxisValue(LayerItem layerItem, double intValue) {
+		Layer layer = mappingLayerItemsToLayers.get(layerItem);
+		if (layer != null){
+			String keyParam = layerItem.isNcWms()?Z_AXIS_NCWMS_KEY_PARAMETER:Z_AXIS_WMS_KEY_PARAMETER;
+			layer.getJSObject().getProperty("params").setProperty(keyParam, intValue);
+			layer.redraw();
+		}
+	}
+
+	/**
+	 * Sets the visibility.
+	 *
+	 * @param layerItem the layer item
+	 * @param visible the visible
+	 */
 	public void setVisibility(LayerItem layerItem, boolean visible) {
 		Layer layer = mappingLayerItemsToLayers.get(layerItem);
 		if (layer != null) {
@@ -479,6 +614,11 @@ public class OpenLayersMap {
 		}
 	}
 
+	/**
+	 * Reproject.
+	 *
+	 * @param proj the proj
+	 */
 	public void reproject(String proj) {
 
 		Constants.log("change " + proj);
@@ -491,22 +631,45 @@ public class OpenLayersMap {
 		mapWidget.getMap().setOptions(mapOptions);
 	}
 
+	/**
+	 * Zoom to max extent.
+	 */
 	public void zoomToMaxExtent() {
 		this.map.zoomToMaxExtent();
 	}
 
+	/**
+	 * Gets the projection.
+	 *
+	 * @return the projection
+	 */
 	public String getProjection() {
 		return this.map.getProjection();
 	}
 
+	/**
+	 * Gets the max extent.
+	 *
+	 * @return the max extent
+	 */
 	public Bounds getMaxExtent() {
 		return this.map.getMaxExtent();
 	}
 
+	/**
+	 * Gets the extent.
+	 *
+	 * @return the extent
+	 */
 	public Bounds getExtent() {
 		return this.map.getExtent();
 	}
 
+	/**
+	 * Activate transect draw.
+	 *
+	 * @param isActivate the is activate
+	 */
 	public void activateTransectDraw(boolean isActivate) {
 		if (isActivate)
 			drawLineControl.activate();
@@ -514,10 +677,20 @@ public class OpenLayersMap {
 			drawLineControl.deactivate();
 	}
 
+	/**
+	 * Activate click data.
+	 *
+	 * @param isActivate the is activate
+	 */
 	public void activateClickData(boolean isActivate) {
 		this.clickData = isActivate;
 	}
-	
+
+	/**
+	 * Activate draw box control.
+	 *
+	 * @param isActivate the is activate
+	 */
 	public void activateDrawBoxControl(boolean isActivate) {
 		if (isActivate)
 			this.drawBoxControl.activate();
@@ -526,6 +699,11 @@ public class OpenLayersMap {
 		vectorLayer.setZIndex(MAX_ZINDEX);
 	}
 
+	/**
+	 * Activate zoom in.
+	 *
+	 * @param isActivate the is activate
+	 */
 	public void activateZoomIn(boolean isActivate) {
 		if (isActivate)
 			zoomBox.activate();
@@ -533,6 +711,11 @@ public class OpenLayersMap {
 			zoomBox.deactivate();
 	}
 
+	/**
+	 * Activate pan.
+	 *
+	 * @param isActivate the is activate
+	 */
 	public void activatePan(boolean isActivate) {
 		if (isActivate)
 			mouseDefaults.activate();
@@ -552,6 +735,11 @@ public class OpenLayersMap {
 	 * //this.map.getLayerByName(name).redraw(true); layer.redraw();
 	 */
 
+	/**
+	 * Removes the cql filter.
+	 *
+	 * @param layerItem the layer item
+	 */
 	public void removeCqlFilter(LayerItem layerItem) {
 		Layer layer = mappingLayerItemsToLayers.get(layerItem);
 
@@ -567,6 +755,12 @@ public class OpenLayersMap {
 		}
 	}
 
+	/**
+	 * Sets the cql filter.
+	 *
+	 * @param layerItem the layer item
+	 * @param filter the filter
+	 */
 	public void setCqlFilter(LayerItem layerItem, String filter) {
 		Layer layer = mappingLayerItemsToLayers.get(layerItem);
 
@@ -574,7 +768,7 @@ public class OpenLayersMap {
 			Constants.log("LAYER : " + layerItem.getName() + " FILTER : " + filter);
 			JSObject layerJs = layer.getJSObject();
 			layerItem.setCqlFilter(filter);
-			
+
 			if (filter.equals("")) {
 				layerJs.getProperty("params").setProperty("CQL_FILTER", "include");
 			} else {
@@ -590,56 +784,113 @@ public class OpenLayersMap {
 		}
 	}
 
+	/**
+	 * Sets the new style.
+	 *
+	 * @param layerItem the layer item
+	 * @param newStyle the new style
+	 */
 	public void setNewStyle(LayerItem layerItem, String newStyle) {
 		Layer layer = mappingLayerItemsToLayers.get(layerItem);
 		layer.getJSObject().getProperty("params").setProperty("STYLES", newStyle);
 		layer.redraw();
 	}
 
+	/**
+	 * Sets the table transect.
+	 *
+	 * @param layerTransect the new table transect
+	 */
 	public void setTableTransect(String layerTransect) {
 		this.tableTransect = layerTransect;
 	}
 
+	/**
+	 * Gets the table transect.
+	 *
+	 * @return the table transect
+	 */
 	public String getTableTransect() {
 		return tableTransect;
 	}
 
+	/**
+	 * Sets the field transect.
+	 *
+	 * @param fieldTransect the new field transect
+	 */
 	public void setFieldTransect(String fieldTransect) {
 		this.fieldTransect = fieldTransect;
 	}
 
+	/**
+	 * Gets the field transect.
+	 *
+	 * @return the field transect
+	 */
 	public String getFieldTransect() {
 		return fieldTransect;
 	}
 
+	/**
+	 * Change size.
+	 *
+	 * @param w the w
+	 * @param h the h
+	 */
 	public void changeSize(String w, String h) {
 		// if together are positives
 		if (Integer.parseInt(w) > 0 && Integer.parseInt(h) > 0) {
+			int height = Integer.parseInt(h) - 10;
 			mapWidget.setWidth(w);
-			mapWidget.setHeight(h);
+			mapWidget.setHeight(height+"");
 			this.updateSize();
 		}
 		// mapWidget.setSize(w, h);
 	}
 
+	/**
+	 * Update size.
+	 */
 	public void updateSize() {
 		map.updateSize();
-		Constants.info("Update size", "(" + getMapWidth() + ","	+ getMapHeight() + ")");
+		GWT.log("Updated size: (" + getMapWidth() + ","	+ getMapHeight() + ")");
 	}
 
+	/**
+	 * Gets the map width.
+	 *
+	 * @return the map width
+	 */
 	public int getMapWidth() {
 		return (int) this.map.getSize().getWidth();
 	}
 
+	/**
+	 * Gets the map height.
+	 *
+	 * @return the map height
+	 */
 	public int getMapHeight() {
 		return (int) this.map.getSize().getHeight();
 	}
 
+	/**
+	 * Gets the map widget.
+	 *
+	 * @return the map widget
+	 */
 	public MapWidget getMapWidget() {
 		return mapWidget;
 	}
 
 
+	/**
+	 * Adds the point.
+	 *
+	 * @param x the x
+	 * @param y the y
+	 */
 	public void addPoint(double x, double y) {
 		// remove any previuos point
 		if (this.prevVf != null)
@@ -654,65 +905,68 @@ public class OpenLayersMap {
 		this.prevVf = vf;
 	}
 
+	/**
+	 * Gets the map view info.
+	 *
+	 * @return the map view info
+	 */
 	public MapViewInfo getMapViewInfo() {
-		double lowerLeftX = (getExtent().getLowerLeftX() < getMaxExtent()
-				.getLowerLeftX()) ? getMaxExtent().getLowerLeftX()
+		double lowerLeftX = getExtent().getLowerLeftX() < getMaxExtent()
+				.getLowerLeftX() ? getMaxExtent().getLowerLeftX()
 				: getExtent().getLowerLeftX();
-		double lowerLeftY = (getExtent().getLowerLeftY() < getMaxExtent()
-				.getLowerLeftY()) ? getMaxExtent().getLowerLeftY()
+		double lowerLeftY = getExtent().getLowerLeftY() < getMaxExtent()
+				.getLowerLeftY() ? getMaxExtent().getLowerLeftY()
 				: getExtent().getLowerLeftY();
-		double upperRightX = (getExtent().getUpperRightX() > getMaxExtent()
-				.getUpperRightX()) ? getMaxExtent().getUpperRightX()
+		double upperRightX = getExtent().getUpperRightX() > getMaxExtent()
+				.getUpperRightX() ? getMaxExtent().getUpperRightX()
 				: getExtent().getUpperRightX();
-		double upperRightY = (getExtent().getUpperRightY() > getMaxExtent()
-				.getUpperRightY()) ? getMaxExtent().getUpperRightY()
+		double upperRightY = getExtent().getUpperRightY() > getMaxExtent()
+				.getUpperRightY() ? getMaxExtent().getUpperRightY()
 				: getExtent().getUpperRightY();
 		return new MapViewInfo(lowerLeftX, lowerLeftY, upperRightX,
 				upperRightY, getMapWidth(), getMapHeight());
 	}
 
+	/**
+	 * Update layers order.
+	 */
 	public void updateLayersOrder() {
-//		System.out.println("LAYERS NEW ORDER");
 		maxZindex = 0;
 		for (Layer layer : baseLayers) {
 			if (Integer.parseInt((String)layer.getZIndex())>maxZindex){
-				
+
 				//ADDED BY FRANCESCO M.
 				int zindex = Integer.parseInt((String)layer.getZIndex());
 				if(zindex>=MAX_ZINDEX){
 					zindex-=-50;
 					GWT.log("zindex to base layer is >=MAX_ZINDEX, recalculate as: "+zindex);
 				}
-				
+
 				maxZindex = zindex;
 				layer.setZIndex(maxZindex);
 				GWT.log("base layer: "+layer.getName()+" have maxZindex is: "+zindex);
-				//OLD CODE
-//				maxZindex = Integer.parseInt((String)layer.getZIndex());
 			}
 		}
 
 		for (LayerItem layerItem : layerItems) {
-//			System.out.println("" + (layerItem.getOrder()+zIndexCounter) + ") "	+ layerItem.getLayer() + " (" + layerItem.getId() + ")");
 			Layer layer = mappingLayerItemsToLayers.get(layerItem);
-			
+
 			//ADDED BY FRANCESCO M.
 			int newOrder = maxZindex + (int)layerItem.getOrder();
-			
+
 			if(newOrder>=MAX_ZINDEX){
-				newOrder-=(layerItems.size()-10);
+				newOrder-=layerItems.size()-10;
 				GWT.log("newOrder>=MAX_ZINDEX is true, recalculate newOrder: "+newOrder);
 			}
 			GWT.log("layer: "+layerItem.getName()+" setting ZIndex: "+newOrder);
 			layer.setZIndex(newOrder);
-		
-			//OLD COD
-//			layer.setZIndex(maxZindex + (int)layerItem.getOrder());
 		}
 	}
 
 	/**
-	 * @param layerItem
+	 * Removes the layer.
+	 *
+	 * @param layerItem the layer item
 	 */
 	public void removeLayer(LayerItem layerItem) {
 		Layer layer = mappingLayerItemsToLayers.get(layerItem);
@@ -722,91 +976,68 @@ public class OpenLayersMap {
 		mappingLayersToLayerItems.remove(layer);
 	}
 
-	
+
+	/**
+	 * Reset base layer to open layer map.
+	 */
 	public void resetBaseLayerToOpenLayerMap(){
 		baseLayersIntefaces.clear();
 	}
-	
-	
+
+
 	/**
-	 * 
+	 * Adds the base layers to open layer map.
+	 *
+	 * @param layers the layers
 	 */
 	public void addBaseLayersToOpenLayerMap(List<GisViewerBaseLayerInterface> layers) {
 
 		if(baseLayersIntefaces==null){
 			baseLayersIntefaces = new ArrayList<GisViewerBaseLayerInterface>();
 		}
-		
+
 		baseLayersIntefaces.addAll(layers);
-		
+
 		for (GisViewerBaseLayerInterface baseLayerInterface : layers) {
 			if(baseLayerInterface.isDisplay())
 				addBaseLayerToOpenLayerMap(baseLayerInterface.getTitle(), baseLayerInterface.getName(), baseLayerInterface.getWmsServiceBaseURL());
 		}
-		
-		
-//		options = new GoogleV3Options();
-//		options.setIsBaseLayer(true);
-//		options.setDisplayInLayerSwitcher(true);
-//		options.setType(GoogleV3MapType.G_TERRAIN_MAP);
-//		googleLayer = new GoogleV3("Google Terrain", options);
-//		map.addLayer(googleLayer);
-//
-		// reproject
-//		MapOptions mapOptions = new MapOptions();
-//		mapOptions.setControls(new JObjectArray(new JSObject[] {}));
-//		mapOptions.setNumZoomLevels(this.numZoomLevels);
-//		mapOptions.setProjection("EPSG:900913");
-//		mapOptions.setMaxExtent(new Bounds(-20037508.34,-20037508.34,20037508.34,20037508.34));
-//		mapOptions.setMaxResolutionToAuto();
-//		mapWidget.getMap().setOptions(mapOptions);
-
-		
-//		addBaseLayer("Blue Marble", "bluemarble_1", "http://iceds.ge.ucl.ac.uk/cgi-bin/icedswms");
-//		addBaseLayerToOpenLayerMap("Global Surface Relief", "etopo1_ice:z", "http://oos.soest.hawaii.edu/erddap/wms/etopo1_ice/request");
-//		addBaseLayerToOpenLayerMap("Shaded Relief", "shaded_relief", "http://maps.ngdc.noaa.gov/soap/etopo1/MapServer/WMSServer");
-//		addBaseLayerToOpenLayerMap("Dem", "dem", "http://maps.ngdc.noaa.gov/soap/etopo1/MapServer/WMSServer");
-//		addBaseLayerToOpenLayerMap("Etopo180", "etopo180:altitude", "http://coastwatch.pfeg.noaa.gov/erddap/wms/erdBAssta5day/request");
-//		addBaseLayerToOpenLayerMap("Groundwater Map", "bgr_groundwater_whymap", "http://www.bgr.de/Service/groundwater/whymap/");
-//		addBaseLayerToOpenLayerMap("LandMask", "LandMask", "http://oos.soest.hawaii.edu/erddap/wms/etopo1_ice/request");
-//		addBaseLayerToOpenLayerMap("Coastlines", "Coastlines", "http://oos.soest.hawaii.edu/erddap/wms/etopo1_ice/request");
-////		addBaseLayer("test2", "NWW3_Global_Best:Tdir", "http://oos.soest.hawaii.edu/erddap/wms/NWW3_Global_Best/request");
-//		addBaseLayerToOpenLayerMap("GEBCO", "GEBCO_08_Grid", "http://www.gebco.net/data_and_products/web_map_service/mapserv/request");
-//		addBaseLayerToOpenLayerMap("test2", "", ""); // do not anything
-
-		//var extent = new OpenLayers.Bounds(-20037508.34,-20037508.34,20037508.34,20037508.34); //-2200000,-712631,3072800,3840000);
-		
-//		map = new OpenLayers.Map( 'map', {
-//			allOverlays: true, 
-//			maxExtent: extent,
-//			projection: '900913' //'EPSG:42304'
-//		});
-		
 	}
 
 	/**
-	 * @param result
+	 * Sets the transect.
+	 *
+	 * @param transect the new transect
 	 */
 	public void setTransect(TransectParameters transect) {
 		this.transect = transect;
 	}
 
+	/**
+	 * Deactivate draw box control.
+	 */
 	public void deactivateDrawBoxControl() {
 		this.drawBoxControl.deactivate();
 		vectorLayer.setZIndex(MAX_ZINDEX);
 	}
-	
+
+	/**
+	 * Activate draw box control.
+	 */
 	public void activateDrawBoxControl() {
 		this.drawBoxControl.activate();
 		vectorLayer.setZIndex(MAX_ZINDEX);
 	}
 
 	/**
-	 * @param title
-	 * @param layerName
-	 * @param geoserverBaseUrl
+	 * Adds the base layer to open layer map.
+	 *
+	 * @param title the title
+	 * @param layerName the layer name
+	 * @param geoserverBaseUrl the geoserver base url
 	 */
 	private void addBaseLayerToOpenLayerMap(String title, String layerName, String geoserverBaseUrl) {
+		GWT.log("Adding base layer title: "+title +", name: "+layerName +", url: "+geoserverBaseUrl);
 		if (geoserverBaseUrl==null || geoserverBaseUrl.equals("") || layerName==null || layerName.equals(""))
 			return;
 		LayerItem l = new LayerItem(true);
@@ -820,21 +1051,30 @@ public class OpenLayersMap {
 	}
 
 	/**
-	 * @param layerItem
-	 * @param displayInLayerSwitcher
+	 * Adds the layer item by wms.
+	 *
+	 * @param layerItem the layer item
+	 * @param displayInLayerSwitcher the display in layer switcher
 	 */
 	public void addLayerItemByWms(LayerItem layerItem, boolean displayInLayerSwitcher) {
 		WMSOptions wmsOptions = new WMSOptions();
 		wmsOptions.setIsBaseLayer(layerItem.isBaseLayer());
 		wmsOptions.setWrapDateLine(true); // default
-//		wmsOptions.setTransitionEffect(TransitionEffect.RESIZE);
 		
+		if(layerItem.isBaseLayer()) {
+			TileOptions tileOpt = new TileOptions();
+			tileOpt.setParameter("tiled", "true");
+			//wmsOptions.setTileSize(new Size(128, 128));
+			wmsOptions.setTileOptions(tileOpt);
+		}
+//		wmsOptions.setTransitionEffect(TransitionEffect.RESIZE);
+
 		WMSParams wmsParams = new WMSParams();
 		wmsParams.setLayers(layerItem.getLayer());
 		wmsParams.setIsTransparent(!layerItem.isBaseLayer());
 		wmsParams.setFormat("image/png");
-		
-		
+
+
 		if(layerItem.getWmsNotStandardParams()!=null){ //LAYER CANNOT BE TILED
 			for (String key : layerItem.getWmsNotStandardParams().keySet()) {
 				String value = layerItem.getWmsNotStandardParams().get(key);
@@ -852,15 +1092,15 @@ public class OpenLayersMap {
 //			wmsParams.setParameter("BBOX", "-415.283203125,-109.951171875,415.283203125,109.951171875");
 		}else //LAYER CAB BE TILED
 			wmsOptions.setBuffer(2); // from serviceimpl
-		
-		
+
+
 		WMS layer = new WMS(layerItem.getTitle(), layerItem.getGeoserverWmsUrl(), wmsParams, wmsOptions);
-		
+
 		layer.setDisplayInLayerSwitcher(displayInLayerSwitcher);
 		mappingLayerItemsToLayers.put(layerItem, layer);
 		mappingLayersToLayerItems.put(layer, layerItem);
 		map.addLayer(layer);
-		
+
 		if (layerItem.isBaseLayer())
 			baseLayers.add(layer);
 		else
@@ -868,7 +1108,9 @@ public class OpenLayersMap {
 	}
 
 	/**
-	 * @return
+	 * Gets the active base layer.
+	 *
+	 * @return the active base layer
 	 */
 	public LayerItem getActiveBaseLayer() {
 		for (Layer l : baseLayers)
@@ -878,7 +1120,7 @@ public class OpenLayersMap {
 	}
 
 	/**
-	 * 
+	 * Zoom out.
 	 */
 	public void zoomOut() {
 		int z = map.getZoom();
@@ -887,7 +1129,7 @@ public class OpenLayersMap {
 	}
 
 	/**
-	 * 
+	 * Removes the data feature.
 	 */
 	public void removeDataFeature() {
 		if (prevVf!=null) {
@@ -895,12 +1137,57 @@ public class OpenLayersMap {
 			prevVf = null;
 		}
 	}
-	
+
 		/**
-	 * @return the map
-	 */
+		 * Gets the map.
+		 *
+		 * @return the map
+		 */
 	public Map getMap() {
 		return map;
 	}
 	
+	
+	/**
+	 * Sets the zoom level.
+	 *
+	 * @param zoomLevel the new zoom level
+	 */
+	public void setZoomLevel(final int zoomLevel) {
+//		GWT.log("setting zoom at: "+zoomLevel);
+//		if(zoomLevel>0 && zoomLevel<numZoomLevels+1) {
+//			map.zoomTo(zoomLevel);
+//		}
+		
+		GWT.log("setting zoom at: "+zoomLevel);
+		Scheduler.get().scheduleDeferred(new ScheduledCommand() {
+			
+			@Override
+			public void execute() {
+				if(zoomLevel>0 && zoomLevel<numZoomLevels+1) {
+					map.zoomTo(zoomLevel);
+				}
+			}
+		});
+	}
+	
+	
+	/**
+	 * Sets the center at.
+	 *
+	 * @param lonLat the new center at
+	 */
+	public void setCenterAt(final LonLat lonLat) {
+
+		Scheduler.get().scheduleDeferred(new ScheduledCommand() {
+			
+			@Override
+			public void execute() {
+				if(lonLat!=null) {
+					map.setCenter(lonLat);
+				}
+			}
+		});
+	}
+
 }
